package ContractReview

import db "ERP_storage/initialize"

func Get_T_audit_name(T_audt int) string {
	switch T_audt {
	case 1:
		return "待提交审核"
	case 2:
		return "待审核"
	case 3:
		return "财务通过"
	case 4:
		return "财务驳回"
	case 5:
		return "总经理通过"
	case 6:
		return "总经理驳回"
	default:
		return ""
	}

}

type ContractReview struct {
	Id                         int     `json:"Id" gorm:"primaryKey;autoIncrement;comment:主键编码"` // 主键编码
	T_name                     string  `json:"T_name" gorm:"size:128"`                          // 项目名称
	T_address                  string  `json:"T_address" gorm:"size:128"`                       // 项目地址
	T_submit                   string  `json:"T_submit" gorm:"size:128"`                        // 合同负责人
	T_predict_sign_time        string  `json:"T_predict_sign_time" gorm:"size:128"`             // 预计签约时间
	T_money                    float64 `json:"T_money" gorm:"type:decimal(10,2)"`               // 总金额
	T_discount_money           float64 `json:"T_discount_money" gorm:"type:decimal(10,2)"`      // 最终优惠金额
	T_have_brokerage_fee       bool    `json:"T_have_brokerage_fee" gorm:"size:4"`              // 是否有居间费
	T_brokerage_fee_money      float64 `json:"T_brokerage_fee_money" gorm:"type:decimal(10,2)"` // 是否有居间费
	T_audit                    int     `json:"T_audit" gorm:"size:4"`                           // 审核状态 待提交审核1 待审核2 财务通过3 财务驳回4 总经理通过5 总经理驳回6
	T_finance_approval_opinion string  `json:"T_finance_approval_opinion" gorm:"size:text"`     // 财务审批意见
	T_manager_approval_opinion string  `json:"T_manager_approval_opinion" gorm:"size:text"`     // 总经理审批意见

	ServiceItemList []ContractReviewServiceItem `json:"ServiceItemList" gorm:"->;foreignkey:T_contract_review_id;references:Id"` // 角色

	T_State    int     `json:"T_State" gorm:"column:t__state;size(2);default(1)"`                  // 0 删除（伪删除）   1 正常
	CreateTime db.Time `json:"CreateTime" gorm:"column:create_time;autoCreateTime;comment:创建时间"`   // 创建时间
	UpdateTime db.Time `json:"UpdateTime" gorm:"column:update_time;autoUpdateTime;comment:最后更新时间"` // 最后更新时间

	T_submit_name string `json:"T_submit_name" gorm:"-"` // 合同负责人名称

}

func (e *ContractReview) TableName() string {
	return "contract_review"
}

type ContractReviewServiceItem struct {
	Id                   int             `json:"Id" gorm:"primaryKey;autoIncrement;comment:主键编码"` // 主键编码
	T_contract_review_id int             `json:"T_contract_review_id" gorm:"size:128"`
	T_service_item_id    int             `json:"T_service_item_id" gorm:"size:128"`      // 服务内容id
	T_quantity           int             `json:"T_quantity" gorm:"size:128"`             // 数量
	T_unit_price         float64         `json:"T_unit_price" gorm:"type:decimal(10,2)"` // 单价
	T_remark             string          `json:"T_remark" gorm:"type:text"`              // 备注
	ServiceItem          ServiceItemOmit `json:"ServiceItem" gorm:"->;foreignkey:T_service_item_id;references:Id"`

	T_State    int     `json:"T_State" gorm:"column:t__state;size(2);default(1)"`         // 0 删除（伪删除）   1 正常
	CreateTime db.Time `json:"-" gorm:"column:create_time;autoCreateTime;comment:创建时间"`   // 创建时间
	UpdateTime db.Time `json:"-" gorm:"column:update_time;autoUpdateTime;comment:最后更新时间"` // 最后更新时间
}

func (e *ContractReviewServiceItem) TableName() string {
	return "contract_review_service_item"
}
